#!/usr/bin/ruby
# coding: utf-8
# $Id: geogroups_children_diff.rb 8204 2023-12-26 21:45:48Z flaterco $

# getTideGeogroupBySid(gt, sbgt, sid, statecode) does interpolation.  This
# script only looks at the geogroups as incoming.  Interpolated stations
# affected by changes to surrounding geogroups will not be detected unless
# and until they are added to a geogroup, at which time they appear as "new."

require "set"
require "json"
require "pg"
require_relative "../import/importNOS_Util"
require_relative "../import/importNOS_JsonReaders"
require_relative "../import/importNOS_Geogroups"

def usagedie
  STDERR.puts "Usage: geogroups_children_diff.rb [--template] old-directory new-directory"
  STDERR.puts "Full paths; each dir must contain all the things."
  exit
end

# -2016      Geogroups not in archive
# 2017-2020  All results concatenated in stations-by-group-tides.json
# 2021-      Results in separate subdirs of groupdir
# This is the old reader from harmbase2-20210110 importNOS_JsonReaders.rb.
# Lotsa duplicated code, but this is all a hack.
def oldReadStationsByGroupTidesJson
  sbgt = File.read("stations-by-group-tides.json")

  # stations-by-group-tides.json is the concatenation of hundreds of separate
  # geogroups queries.  We don't need to keep the queries separated, but we
  # do need to make it parse.
  sbgt.gsub!("    }\n  ]\n}{\n  \"stationList\": [", "    },")

  sbgt_array = JSON.parse(sbgt)["stationList"]
  sbgt = Hash.new
  sbgt_array.each{|sta|
    sid = sta["stationId"]
    # The geogroups themselves are in there with null sids.
    unless sid.nil?
      if sbgt.include?(sid)
	exist = sbgt[sid]

	# Special case fixup, 2017
	# Station 8730667, Alabama Point, appears in both Florida and Alabama.
	# 30.2786° N,  87.5550° W
	# Old name:  Alabama Point, Perdido Pass, Alabama
	# 1762: Perdido Bay, Gulf Coast-cont., Florida
	# 1392: Alabama
	# It is, in fact, just barely in Alabama.  The old name is accurate.
	if sid == "8730667"
	  sbgt[sid] = sta if sta["parentGeoGroupId"] == "1392"
	  next
	end

	# Verify that *only* the level field may disagree in duplicates.
	sta.each {|k,v|
	  unless k == "level" or exist[k] == v
	    print sid, " ", k, " ", v, " != ", exist[k], "\n"
	    raise "Unhandled: dup conflict in osbgt"
	  end
	}
      else
	sbgt[sid] = sta
      end
    end
  }
  sbgt
end

# Dispatch to whichever version of the reader is applicable
def readStationsByGroupTidesSwitch
  if File.exist?('geogroups')
    readStationsByGroupTidesJson
  elsif File.exist?('stations-by-group-tides.json')
    oldReadStationsByGroupTidesJson
  else
    raise 'Neither old nor new geogroups found in directory'
  end
end

if ARGV.length == 3 && ARGV[0] == '--template'
  $template=true
  olddir = ARGV[1]
  newdir = ARGV[2]
elsif ARGV.length == 2
  $template=false
  olddir = ARGV[0]
  newdir = ARGV[1]
else
  usagedie
end
Dir.chdir olddir
gt_old = readGeogroupsTidesJson
sbgt_old = readStationsByGroupTidesSwitch
Dir.chdir newdir
gt_new = readGeogroupsTidesJson
sbgt_new = readStationsByGroupTidesSwitch

sids_old = sbgt_old.keys.to_set
sids_new = sbgt_new.keys.to_set
sids_only_old = sids_old - sids_new
sids_only_new = sids_new - sids_old
sids_both = sids_old & sids_new

def putz(s)
  puts ($template ? '-- ' : '') + s
end

def generate_template(db,sid)
  if $template
    db.exec("select name, lat, lng from data_sets_old" \
            " where station_id = $1", [sid]).each_row {|row|
      name, lat, lng = row
      print "-- #{sid} #{lat} #{lng}\n"
      print "update data_sets_old\n"
      print "  set name = '#{name}'\n  where name = '#{name}';\n"
    }
  end
end

# When --template, don't print anything if the station doesn't exist in
# data_sets_old.
def station_exists(db,sid)
  query = db.exec("select count(*) from data_sets_old where station_id = $1", [sid])
  !query.getvalue(0,0).to_i.zero?
end

db = PG::Connection.new(:dbname => 'harmbase2') if $template
putz "Old = #{olddir}"
putz "New = #{newdir}"
unless $template
  # Nothing to be done with these?
  sids_only_old.each{|sid|
    putz "Station #{sid} only in old (disappeared):"
    ggid_old = sbgt_old[sid]["parentGeoGroupId"].to_i
    putz "  Old geogroup #{ggid_old}"
    gold1 = gt_old[ggid_old]
    gold2 = getTideGeogroupsByGgid(gt_old,ggid_old)
    putz "  Old #{gold1}"
    putz "  Old #{gold2}"
  }
end
sids_only_new.each{|sid|
  if !$template || station_exists(db,sid)
    putz "Station #{sid} only in new (added):"
    ggid_new = sbgt_new[sid]["parentGeoGroupId"].to_i
    putz "  New geogroup #{ggid_new}"
    gnew1 = gt_new[ggid_new]
    gnew2 = getTideGeogroupsByGgid(gt_new,ggid_new)
    putz "  New #{gnew1}"
    putz "  New #{gnew2}"
    generate_template(db,sid)
  end
}
sids_both.each{|sid|
  if !$template || station_exists(db,sid)
    ggid_old = sbgt_old[sid]["parentGeoGroupId"].to_i
    ggid_new = sbgt_new[sid]["parentGeoGroupId"].to_i
    gold1 = gt_old[ggid_old]
    gold2 = getTideGeogroupsByGgid(gt_old,ggid_old)
    gnew1 = gt_new[ggid_new]
    gnew2 = getTideGeogroupsByGgid(gt_new,ggid_new)
    if ggid_old != ggid_new
      putz "Station #{sid} MOVED:"
      putz "  Old geogroup #{ggid_old}"
      putz "  Old #{gold1}"
      putz "  Old #{gold2}"
      putz "  New geogroup #{ggid_new}"
      putz "  New #{gnew1}"
      putz "  New #{gnew2}"
      generate_template(db,sid)
    elsif gold1 != gnew1 || gold2 != gnew2
      putz "Station #{sid}'s geogroup changed:"
      putz "  Old geogroup #{ggid_old}"
      putz "  Old #{gold1}"
      putz "  Old #{gold2}"
      putz "  New geogroup #{ggid_new}"
      putz "  New #{gnew1}"
      putz "  New #{gnew2}"
      generate_template(db,sid)
    end
  end
}
